/******************************************************************************
 * @file     nano1xx_timer.c
 * @brief    NANO1xx TIMER driver source file
 * @version  1.0.1
 * @date     04, September, 2012
 *
 * @note
 * Copyright (C) 2012-2014 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#include "nano1xx_sys.h"
#include "nano1xx_assert.h"


/** @addtogroup NANO1xx_Device_Driver NANO1xx Device Driver
  @{
*/

/** @addtogroup NANO1xx_TIMER_Driver TIMER Driver
  @{
*/


/** @addtogroup NANO1xx_TIMER_EXPORTED_FUNCTIONS TIMER Exported Functions
  @{
*/

/**
  * @brief  Initial timer channel to work in specified mode.
  * @param[in]  timer Timer channel to configure
  * @param[in]  u8Prescale Prescale value
  * @param[in]  u32Compare Comparator value, valid valies are between 2~0xFFFFFF
  * @param[in]  u32Mode Timer operating mode. Valid modes are
  *             - \ref TIMER_CTL_MODESEL_ONESHOT
  *             - \ref TIMER_CTL_MODESEL_PERIODIC
  *             - \ref TIMER_CTL_MODESEL_TOGGLE
  *             - \ref TIMER_CTL_MODESEL_CONTINUOUS
  * @return None
  */
void TIMER_Init(TIMER_TypeDef *timer, uint8_t u8Prescale, uint32_t u32Compare, uint32_t u32Mode)
{
	assert_param((u32Compare > 1) && (u32Compare < 0x1000000));
	assert_param((u32Mode & ~TIMER_CTL_MODESEL_MASK) == 0);

	if(timer == TIMER0)
		CLK->APBCLK |= CLK_APBCLK_TMR0_EN;
	else if(timer == TIMER1)
		CLK->APBCLK |= CLK_APBCLK_TMR1_EN;
	else if(timer == TIMER2)
		CLK->APBCLK |= CLK_APBCLK_TMR2_EN;
	else
		CLK->APBCLK |= CLK_APBCLK_TMR3_EN;

	timer->CTL = 0;
	timer->ISR = (TIMER_ISR_TMRIS | TIMER_ISR_TCAPIS | TIMER_ISR_TMRWAKESTS | TIMER_ISR_NCAPDETSTS);
	timer->PRECNT = u8Prescale;
	timer->CMPR = u32Compare;
	timer->CTL = u32Mode;
        return;
}

/**
  * @brief  De-init timer channel. Close module code and disable interrupt.
  * @param[in]  timer Timer channel to disable
  * @return None
  */
void TIMER_DeInit(TIMER_TypeDef *timer)
{
	timer->CTL = TIMER_CTL_SWRST;
	while(timer->CTL & TIMER_CTL_SWRST);
	timer->IER = 0;

	if(timer == TIMER0) {
		CLK->APBCLK &= ~CLK_APBCLK_TMR0_EN;
		NVIC_DisableIRQ(TMR0_IRQn);
	} else if(timer == TIMER1) {
		CLK->APBCLK &= ~CLK_APBCLK_TMR1_EN;
		NVIC_DisableIRQ(TMR1_IRQn);
	} else if(timer == TIMER2) {
		CLK->APBCLK &= ~CLK_APBCLK_TMR2_EN;
		NVIC_DisableIRQ(TMR2_IRQn);
	} else {
		CLK->APBCLK &= ~CLK_APBCLK_TMR3_EN;
		NVIC_DisableIRQ(TMR3_IRQn);
	}

}

/**
  * @brief  Enable timer interrupt
  * @param[in]  timer Timer channel to configure
  * @param[in]  u32Mask Interrupt mask to enable, a combination of \ref TIMER_IER_TMRIE and \ref TIMER_IER_TMRIE
  * @return None
  */
void TIMER_EnableInt(TIMER_TypeDef *timer, uint32_t u32Mask)
{
        assert_param(u32Mask != 0);
        assert_param((u32Mask & ~(TIMER_IER_TMRIE | TIMER_IER_TCAPIE)) == 0);

        timer->ISR = timer->ISR;  // clear pending interrupt
        timer->IER |= u32Mask;

	if(timer == TIMER0) {
		NVIC_EnableIRQ(TMR0_IRQn);
	} else if(timer == TIMER1) {
		NVIC_EnableIRQ(TMR1_IRQn);
	} else if(timer == TIMER2) {
		NVIC_EnableIRQ(TMR2_IRQn);
	} else {
		NVIC_EnableIRQ(TMR3_IRQn);
	}
}

/**
  * @brief  Disable timer interrupt
  * @param[in]  timer Timer channel to configure
  * @param[in]  u32Mask Interrupt mask to Disable, a combination of \ref TIMER_IER_TMRIE and \ref TIMER_IER_TMRIE
  * @return None
  */
void TIMER_DisableInt(TIMER_TypeDef *timer, uint32_t u32Mask)
{
        assert_param(u32Mask != 0);
        assert_param((u32Mask & ~(TIMER_IER_TMRIE | TIMER_IER_TCAPIE)) == 0);

	timer->IER &= ~u32Mask;

        if(timer->IER == 0) {
		if(timer == TIMER0) {
			NVIC_DisableIRQ(TMR0_IRQn);
		} else if(timer == TIMER1) {
			NVIC_DisableIRQ(TMR1_IRQn);
		} else if(timer == TIMER2) {
			NVIC_DisableIRQ(TMR2_IRQn);
		} else {
			NVIC_DisableIRQ(TMR3_IRQn);
		}
        }

}


/**
  * @brief  Configure TIMERx trigger event
  * @param[in]  timer Timer channel
  * @param[in]  u32Event IP to be trigger, could a combination of \ref TIMER_CTL_ADCTEEN, \ref TIMER_CTL_DACTEEN, and \ref TIMER_CTL_PDMA_TEEN
  * @param[in]  u32Src Source to trigger ADC/DAC/PDMA, could be ether \ref TIMER_CTL_CAPTRGEN or \ref TIMER_CTL_TIMEOUTTRGEN.
  * @return None
  */
void TIMER_ConfigTriggerEvent(TIMER_TypeDef *timer, uint32_t u32Event, uint32_t u32Src)
{
	uint32_t reg;

        assert_param(u32Event != 0);
        assert_param((u32Event & ~(TIMER_CTL_ADCTEEN | TIMER_CTL_DACTEEN | TIMER_CTL_PDMA_TEEN)) == 0);

	assert_param((u32Src == TIMER_CTL_CAPTRGEN) || (u32Src == TIMER_CTL_TIMEOUTTRGEN));

	reg = timer->CTL & ~(TIMER_CTL_ADCTEEN | TIMER_CTL_DACTEEN | TIMER_CTL_PDMA_TEEN | TIMER_CTL_CAPTRGEN);
        timer->CTL = reg | u32Event | u32Src;
}





/**
  * @brief  Initialize WDT
  * @param[in]  u32Interval Watch Dog Timer time out interval, valid values are:
  *             - \ref WDT_CTL_WTIS_2POW4
  *             - \ref WDT_CTL_WTIS_2POW6
  *             - \ref WDT_CTL_WTIS_2POW8
  *             - \ref WDT_CTL_WTIS_2POW10
  *             - \ref WDT_CTL_WTIS_2POW12
  *             - \ref WDT_CTL_WTIS_2POW14
  *             - \ref WDT_CTL_WTIS_2POW16
  *             - \ref WDT_CTL_WTIS_2POW18  
  * @param[in]  u32Delay Watch Dog Timer reset delay, valid values are:
  *             - \ref WDT_CTL_WTRDSEL_1026CLK
  *             - \ref WDT_CTL_WTRDSEL_130CLK
  *             - \ref WDT_CTL_WTRDSEL_18CLK
  *             - \ref WDT_CTL_WTRDSEL_3CLK  
  * @return None
  * @note This API does not start Watch Dog Timer
  */
void WDT_Init(uint32_t u32Interval, uint32_t u32Delay)
{
	uint32_t reg;
	assert_param((u32Interval & ~WDT_CTL_WTIS_MASK) == 0);
	assert_param((u32Delay & ~WDT_CTL_WTRDSEL_MASK) == 0);

        CLK->APBCLK |= CLK_APBCLK_WDT_EN;
        reg = WDT->CTL;
        reg &= ~(WDT_CTL_WTIS_MASK | WDT_CTL_WTRDSEL_MASK);
        WDT->CTL = reg | u32Interval | u32Delay;
}

/**
  * @brief  Disable WDT. Close module clock and disable WDT interrupt
  * @param  None
  * @return None
  */
void WDT_DeInit(void)
{

	WDT->CTL &= ~WDT_CTL_WTE;
	WDT->IER &= ~WDT_IER_WDTIE;

	if(!(WWDT->IER & WWDT_IER_IE))
		NVIC_DisableIRQ(WDT_IRQn);

        CLK->APBCLK &= ~CLK_APBCLK_WDT_EN;

}

/**
  * @brief  Enable WDT interrupt
  * @param  None
  * @return None
  * @note   This API will clear pending interrupt status before enabled interrupt
  */
void WDT_EnableInt(void)
{
	WDT->ISR = WDT_ISR_WDTIS;
	WDT->IER |= WDT_IER_WDTIE;
        NVIC_EnableIRQ(WDT_IRQn);

}

/**
  * @brief  Disable WDT interrupt
  * @param  None
  * @return None
  */
void WDT_DisableInt(void)
{
	WDT->IER &= ~WDT_IER_WDTIE;
	if(!(WWDT->IER & WWDT_IER_IE))
		NVIC_DisableIRQ(WDT_IRQn);

}


/**
  * @brief  Enables WWDT clock, configure prescale and comparator
  * @param[in]  u32Prescale WWDT clock prescale, valid values are:
  *                 - \ref WWDT_CR_PERIODSEL_1
  *                 - \ref WWDT_CR_PERIODSEL_2
  *                 - \ref WWDT_CR_PERIODSEL_4
  *                 - \ref WWDT_CR_PERIODSEL_8
  *                 - \ref WWDT_CR_PERIODSEL_16
  *                 - \ref WWDT_CR_PERIODSEL_32
  *                 - \ref WWDT_CR_PERIODSEL_64
  *                 - \ref WWDT_CR_PERIODSEL_128
  *                 - \ref WWDT_CR_PERIODSEL_192
  *                 - \ref WWDT_CR_PERIODSEL_256
  *                 - \ref WWDT_CR_PERIODSEL_384
  *                 - \ref WWDT_CR_PERIODSEL_512
  *                 - \ref WWDT_CR_PERIODSEL_768
  *                 - \ref WWDT_CR_PERIODSEL_1024
  *                 - \ref WWDT_CR_PERIODSEL_1536
  *                 - \ref WWDT_CR_PERIODSEL_2048  
  * @param[in]  u8Cmp Comparator value. Valid range are between 63 and 0
  * @return None
  * @note   This API can only call once after systme boot up
  */
void WWDT_Init(uint32_t u32Prescale, uint8_t u8Cmp)
{
        assert_param(u8Cmp < 64);
        assert_param((u32Prescale & ~WWDT_CR_PERIODSEL_MASK) == 0);

        CLK->APBCLK |= CLK_APBCLK_WDT_EN;
        WWDT->CR = u32Prescale | (u8Cmp << 16) | 1;

        return;
}

/**
  * @brief  Sorry, can't do so once WWDT init.
  * @param  none
  * @param  none
  */
void WWDT_DeInit(void)
{
        return;
}


/**
  * @brief  Check if WWDT is CPU reset source or not
  * @param  none
  * @return Yes or no.
  * @retval 0 WWDT is not CPU reset source
  * @retval 1 WWDT is CPU reset source
  * @note   This API will clear WWDT reset status
  */
uint8_t WWDT_IsResetSource(void)
{
        int8_t ret = (WWDT->STS & WWDT_STS_RF) ? 1 : 0;

        if(ret)
                WWDT->STS = WWDT_STS_RF;

        return(ret);

}


/**
  * @brief  Enable WWDT interrupt
  * @param  None
  * @return None
  * @note   This API will clear pending interrupt status before enabled interrupt
  */
void WWDT_EnableInterrupt(void)
{

        WWDT->STS = WWDT_STS_IF;	// clear pending interrupt, if any
        WWDT->IER = WWDT_IER_IE;

        NVIC_EnableIRQ(WDT_IRQn);

        return;
}


/*@}*/ /* end of group NANO1xx_TIMER_EXPORTED_FUNCTIONS */

/*@}*/ /* end of group NANO1xx_TIMER_Driver */

/*@}*/ /* end of group NANO1xx_Device_Driver */

/*** (C) COPYRIGHT 2012 Nuvoton Technology Corp. ***/

